"""
Blank URLConf just to keep runtests.py happy.
"""
from rest_framework.compat import patterns, url, include
import nodetree

urlpatterns = patterns('',
#    url(r'^nodetree/', include(nodetree.app.urls)),
)
